package com.newxtc.fw.client;

import java.util.LinkedHashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.newxtc.fw.client.util.HttpClient;
import com.newxtc.fw.client.util.JsonParamUtil;

public class SmsBussi {
	private final static Logger logger = LoggerFactory.getLogger(SmsBussi.class);

	/**
	 * 
	 * @param phone
	 * @param msg
	 * @return
	 */
	public static JSONObject sendMsg(String phone, String msg) {
		return sendMsgSp(phone, msg, null);
	}

	/**
	 * 发送短信 (指定SP)
	 */
	public static JSONObject sendMsgSp(String phone, String msg, String spCode) {
		long beginTime = System.currentTimeMillis();
		Map<String, Object> paramMap = new LinkedHashMap<>();
		paramMap.put("phone", phone);
		paramMap.put("msg", msg);
		if (spCode != null) {
			paramMap.put("spCode", spCode);
		}
		Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
		String jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", "SmsBussiServlet", signParam, 1000) : null;
		try {
			JSONObject jsonRet = (jsonResp != null && jsonResp.startsWith("{") && jsonResp.endsWith("}")) ? JSONObject.parseObject(jsonResp) : null;
			if (jsonRet != null) {
				String ret = jsonRet.getString("ret");
				if (ret != null && "0".equals(ret)) {
					String retSpCode = (jsonRet != null) ? jsonRet.getString("spCode") : null;
					long cost = System.currentTimeMillis() - beginTime;
					logger.info("sendMsg() spCode=" + retSpCode + ",phone=" + phone + ",msg=" + msg + ",cost=" + cost);
				} else {
					logger.error("sendMsg() ret=" + ret + ",jsonResp=" + jsonResp);
				}
				return jsonRet;
			} else {
				logger.error("sendMsg() paramsMap=" + paramMap + ",jsonResp=" + jsonResp);
				return null;
			}
		} catch (Exception e) {
			logger.error("sendMsg() paramsMap=" + paramMap + ",jsonResp=" + jsonResp);
			return null;
		}
	}

	/**
	 * 发送模板短信
	 */
	public static JSONObject sendTpMsg(String phone, String templateId, Map<String, String> templateMap) {
		return sendTpMsgSp(phone, templateId, templateMap, null);
	}

	/**
	 * 发送模板短信 (指定SP)
	 * 
	 * @param phone
	 * @param templateId
	 * @param templateMap
	 * @param spCode
	 * @return
	 */
	public static JSONObject sendTpMsgSp(String phone, String templateId, Map<String, String> templateMap, String spCode) {
		if (phone != null && phone.length() == 11) {
			String jsonResp = null;
			Map<String, Object> paramMap = new LinkedHashMap<>();
			try {
				long beginTime = System.currentTimeMillis();
				String templateJson = (templateMap != null) ? JSONObject.toJSONString(templateMap) : null;
				paramMap.put("phone", phone);
				paramMap.put("templateId", templateId);
				paramMap.put("templateJson", templateJson);
				if (spCode != null) {
					paramMap.put("spCode", spCode);
				}
				Map<String, Object> signParam = JsonParamUtil.getSignParam(paramMap);
				jsonResp = (signParam != null) ? HttpClient.execClient("fireWareUrl", "SmsBussiServlet", signParam, 1000) : null;
				JSONObject jsonRet = (jsonResp != null && jsonResp.startsWith("{") && jsonResp.endsWith("}")) ? JSONObject.parseObject(jsonResp) : null;
				if (jsonRet != null) {
					String ret = jsonRet.getString("ret");
					if (ret != null && "0".equals(ret)) {
						String retSpCode = (jsonRet != null) ? jsonRet.getString("spCode") : null;
						long cost = System.currentTimeMillis() - beginTime;
						logger.info("sendTpMsg() spCode=" + retSpCode + ",phone=" + phone + ",templateId=" + templateId + ",templateJson=" + templateJson + ",cost=" + cost);
						return jsonRet;
					} else {
						logger.error("sendTpMsg() ret=" + ret + ",jsonResp=" + jsonResp);
						return null;
					}
				} else {
					logger.error("sendTpMsg() paramsMap=" + paramMap + ",jsonResp=" + jsonResp);
					return null;
				}
			} catch (Exception e) {
				logger.error("sendTpMsg() paramsMap=" + paramMap + ",jsonResp=" + jsonResp);
				return null;
			}
		} else {
			logger.error("sendTpMsgSp() phone=" + phone);
			return null;
		}
	}
}
